#' Download population arrangements in Brazil
#'
#' @description
#' This function reads the official data on population arrangements (Arranjos
#' Populacionais) of Brazil. Original data were generated by the Institute of
#' Geography and Statistics (IBGE)  For more information about the methodology,
#' see details at \url{https://www.ibge.gov.br/apps/arranjos_populacionais/2015/pdf/publicacao.pdf}
#'
#' @param year A year number in YYYY format. Defaults to `2015`
#' @param simplified Logic `FALSE` or `TRUE`, indicating whether the function returns
#' the data set with 'original' resolution or a data set with 'simplified' borders.
#' Defaults to `TRUE`. For spatial analysis and statistics users should set
#' `simplified = FALSE`. Borders have been simplified by removing vertices of
#' borders using `st_simplify{sf}` preserving topology with a `dTolerance` of 100.
#' @param showProgress Logical. Defaults to `TRUE` display progress bar
#'
#' @return An `"sf" "data.frame"` object
#'
#' @export
#' @examples \dontrun{ if (interactive()) {
#' # Read urban footprint of Brazilian cities in an specific year
#' uc <- read_pop_arrangements(year=2015)
#' } }
read_pop_arrangements <- function(year=2015, simplified=TRUE, showProgress=TRUE){

  # Get metadata with data url addresses
  temp_meta <- select_metadata(geography="pop_arrengements", year=year, simplified=simplified)

  # list paths of files to download
  file_url <- as.character(temp_meta$download_path)

  # download files
  temp_sf <- download_gpkg(file_url, progress_bar = showProgress)
  return(temp_sf)

}
